#ifndef WIN10VT_H_INCLUDED__
#define WIN10VT_H_INCLUDED__

#ifdef __cplusplus
extern "C" {
#endif

/** \brief  Function-like macros to simplify the creation of VT sequences as ANSI or
 *          wide string literals, respectively.
 *          For supported sequences refer to:
 *          https://docs.microsoft.com/en-us/windows/console/console-virtual-terminal-sequences */
#define ESC(seq) "\x1B" #seq
#define ESCW(seq) L"\x1B" L ## #seq

/** brief  Flags representing the console handles.
 *         NOTE: VT_CONIN might be referenced by stdin
 *               while VT_CONOUT might be referenced by both stdout and stderr.
 *               There is no separate console handle for stderr. */
#define VT_CONNONE (0U)
#define VT_CONIN   (1U)
#define VT_CONOUT  (2U)
#define VT_CONALL  (VT_CONIN | VT_CONOUT)

/** \brief  Enable or disable Virtual Terminal processing for the console window.
 * \param  f_use     Flags referencing the handles to be updated. One or more out of VT_CONIN, VT_CONOUT.
 * \param  f_enable  Flags out of f_use where VT processing shall be enabled. VT processing will be
 *                   disabled for handles where flags are set in f_use but not in f_enable.
 * \return  Flags which represent the console handles where VT processing was updated.
 *          That is, either none (VT_CONNONE), one, or both of VT_CONIN, VT_CONOUT.
 *          NOTE: Not being able to update the console handles may indicate that either no console window
 *                is attached, or that a legacy console is attached where VT processing is not supported. */
unsigned SetVTConMode(unsigned f_use, unsigned f_enable);

/** brief  Flags representing the standard handles.
 *         NOTE: VT_STDIN may refer to CONIN$ and
 *               both VT_STDOUT and VT_STDERR may refer to CONOUT$.
 *               There is no separate console device for VT_STDERR. */
#define VT_STDNONE (0U)
#define VT_STDIN   (1U)
#define VT_STDOUT  (2U)
#define VT_STDERR  (4U)
#define VT_STDALL  (VT_STDIN | VT_STDOUT | VT_STDERR)

/** \brief  Determine which of the standard handles refer to a console.
 * \return  Flags which represent the standard handles that refer to a console.
 *          That is, either none (VT_STDNONE), one, or more out of VT_STDIN, VT_STDOUT, VT_STDERR. */
unsigned GetStdReferCon(void);

#ifdef __cplusplus
}
#endif

#endif // WIN10VT_H_INCLUDED__
